#right whale risk reduction analysis
require('truncnorm')
require('mvtnorm')
require('ggplot2')
set.seed(8525754)  #initialize random number generator so we can repeat analysis 
#1) load data sets

Strikes<-read.csv(file='StrikeSpeed.csv')  #for logistic regression; available from second author
Transits<-read.csv(file='TransitSpeed.csv')  #available from second author
#remove unneeded rows
Transits<-Transits[,c(2,4,5,7,10)]
#remove transits for which transit length=0
Transits=Transits[-which(Transits[,"transit_length"]==0),]
#remove transits for which speed=0
Transits=Transits[-which(Transits[,"transit_avg_sog"]<2),]
#transit dataset for strike analysis
Transit2<-read.csv('20121123_risk_reduction.csv')  #load transit data for multiple vessel types for strike rate analysis
Transit2<-Transit2[Transit2[,"speed"]>2,]


#1) Fit simple logistic regression models to strike data
Fate.df=data.frame(cbind(Strikes[,"Fate"],Strikes[,"Speed"],Strikes[,"Speed"]^2))
colnames(Fate.df)=c('Fate','Speed','Speed2')
log.reg<-glm(Fate~Speed,data=Fate.df,family='binomial')  #smaller aic for simpler model
log.reg2<-glm(Fate~Speed+Speed2,data=Fate.df,family='binomial')

Pred.df=data.frame(matrix(1,26,2))
Pred.df[,2]=c(0:25)
colnames(Pred.df)=c('Fate','Speed')
Preds=cbind(Pred.df,predict(log.reg,newdata=Pred.df,type="response",se=TRUE))

#2) Fit Bayesian probit regression model to strike/lethality data
Pred.df=as.matrix(Pred.df)
Fate.df=as.matrix(Fate.df[,c(1:2)])
X=matrix(1,nrow(Fate.df),2)
X[,2]=Fate.df[,2]
n.iter=11000
burnin=1000
Beta=matrix(0,2,n.iter)
Y.tilde=rep(0,nrow(Fate.df))
XpXinv=solve(t(X)%*%X)
Pred.Bayes=matrix(0,26,n.iter)
n.pred=nrow(Pred.df)
for(iiter in 2:n.iter){
  if(iiter%%1000==1)cat(paste("Iteration",iiter,"of",n.iter,"\n"))
  Lin.pred=X%*%Beta[,iiter-1]
  for(irec in 1:nrow(Fate.df)){  #could probably do this a lot quicker with an apply
    if(Fate.df[irec,1]==0)Y.tilde[irec]=rtruncnorm(1,a=-Inf,b=0,mean=Lin.pred[irec],sd=1)
    else Y.tilde[irec]=rtruncnorm(1,a=0,b=Inf,mean=Lin.pred[irec],sd=1)
  }
  Beta[,iiter]=rmvnorm(1,XpXinv%*%t(X)%*%Y.tilde,XpXinv)
  Pred.Bayes[,iiter]=pnorm(Pred.df%*%matrix(Beta[,iiter],2,1))
}
Pred.Bayes=Pred.Bayes[,(burnin+1):n.iter]
Beta=Beta[,(burnin+1):n.iter]

#calculate some summary statistics and plot
UpperCI=apply(Pred.Bayes,1,'quantile',0.975)
LowerCI=apply(Pred.Bayes,1,'quantile',0.025)
PostMean=apply(Pred.Bayes,1,'mean')

tiff('ConnMs13-0004_fig3.tif',width=3.9,height=3.9,units='in',compression="lzw",res=250)
par(las=1)
plot(Pred.df[,2],PostMean,type="l",lwd=3,xlab="Strike speed (knots)",ylab="Pr(Lethal strike)",cex.axis=1.3,cex.lab=1.3,ylim=c(0,1))
polygon(c(0,Pred.df[,2],25),y=c(0,UpperCI,0),col="grey",border=NA)
polygon(c(0,Pred.df[,2],25),y=c(0,LowerCI,0),col="white",border=NA)
lines(Pred.df[,2],PostMean,lwd=3)
lines(Pred.df[,2],Preds[,"fit"],lty=2,lwd=3)
lines(Pred.df[,2],1/(1+exp(4.89-0.41*Pred.df[,2])),lty=3,lwd=3)
dev.off()


n.transits=10000

List.names1=c("Cargo","Passenger","Sovereign")
List.names2=c("N","Y")
Tspeed=vector("list",3)
Sspeed=Tspeed
for(i in 1:3){
  Tspeed[[i]]=vector("list",2)
  Sspeed[[i]]=vector("list",2)
  for(j in 1:2){
    Cur.dat=Transit2[which(Transit2[,"vessel_type"]==List.names1[i] & Transit2[,"sma_active"]==List.names2[j]),]
    Tspeed[[i]][[j]]=sample(Cur.dat[,"speed"],n.transits,replace=TRUE,prob=Cur.dat[,"transit_num_recs"])
  }
}
#actual strike speeds
Sspeed[[1]][[1]]=22
Sspeed[[1]][[2]]=NA
Sspeed[[2]][[1]]=20.8
Sspeed[[2]][[2]]=NA
Sspeed[[3]][[1]]=c(13,13,15,20,15,15,22,14)
Sspeed[[3]][[2]]=c(8.5,12)

#plot histogram of random speeds with observed strike speeds overlayed
Tspeed.df=data.frame(matrix(0,n.transits*6,3))
Temp=NA
Col1=NA
Col2=NA
for(i in 1:3){
  for(j in 1:2){
    Temp=c(Temp,Tspeed[[i]][[j]])
    Col1=c(Col1,rep(List.names1[i],n.transits))
    Col2=c(Col2,rep(List.names2[j],n.transits))
  }
}
Tspeed.df[,1]=Temp[-1]
Tspeed.df[,2]=Col1[-1]
Tspeed.df[,3]=Col2[-1]
colnames(Tspeed.df)=c("Speed","Vessel.type","SMA.active")
Sspeed.df=data.frame(matrix(0,12,3))
Sspeed.df[,1]=c(22,20.8,13,13,15,20,15,15,22,14,8.5,12)
Sspeed.df[,2]=c("Cargo","Passenger",rep("Sovereign",10))
Sspeed.df[,3]=c(rep("N",10),"Y","Y")
colnames(Sspeed.df)=colnames(Tspeed.df)
cbbPalette <- c("#000000", "#E69F00", "#56B4E9", "#009E73", "#F0E442", "#0072B2", "#D55E00", "#CC79A7")

tiff('ConnMS13-00004_fig2.tif',height=8,width=8,units='in',compression="lzw",res=250)
plot1=ggplot(Tspeed.df) + geom_line(aes(x = Speed),stat='density',size=1) + labs(x = 'Vessel speed (knots)',y="Probability density") + facet_wrap(~SMA.active+Vessel.type,scales="free_y")+scale_colour_manual(values=cbbPalette)
plot1=plot1+xlim(0,35)+geom_point(data=Sspeed.df,aes(x=Speed,y=0),col='red',shape='x',size=6)
plot1+opts(axis.text.x=theme_text(size=15),axis.text.y=theme_text(size=15),strip.text.x=theme_text(size=16),legend.text=theme_text(size=16),axis.title.x=theme_text(size=18),axis.title.y=theme_text(size=18,angle=90),legend.title=theme_text(size=16))
dev.off()

#Tspeed=sample(Transits[Transits[,"sma_status"]=="Inactive","transit_avg_sog"],n.transits,replace=TRUE)  #speed data from control transits

#this commented out bit will simulate some data for testing
#n.sim=10000
#Tspeed=rnorm(n.sim,10,2.5)
#p=1-exp(-exp(-6+Tspeed*0.3))
#y=rbinom(n.sim,1,p)
#Sspeed=Tspeed[y==1]
niter=110000
set.seed(12345)
burnin=10000
MCMC=matrix(0,2,niter)
MCMC[1,1]=-6
MCMC[2,1]=0.35
sd.1=5
sd.2=.08
logL<-function(par,degree,Tspeed,Sspeed){ #degree gives degree of polynomial
  ll=0
  if(degree==1){
    alpha0=par[1]
    alpha1=par[2]
    for(i in 1:3){
      for(j in 1:2){
        if(is.na(Sspeed[[i]][[j]][1])==FALSE){
          norm.const=mean((1-exp(-exp(alpha0+alpha1*Tspeed[[i]][[j]]))))+.000001
          ll=ll+sum(log(1-exp(-exp(alpha0+alpha1*Sspeed[[i]][[j]]))))-length(Sspeed[[i]][[j]])*log(norm.const)          
        }
      }
    }
  }
  if(degree==2){
    alpha0=par[1]
    alpha1=par[2]
    alpha2=par[3]
    for(i in 1:3){
      for(j in 1:2){
        if(is.na(Sspeed[[i]][[j]][1])==FALSE){
          norm.const=mean((1-exp(-exp(alpha0+alpha1*Tspeed[[i]][[j]]+alpha2*Tspeed[[i]][[j]]^2))))+.000001
          ll=ll+sum(log(1-exp(-exp(alpha0+alpha1*Sspeed[[i]][[j]]+alpha2*Sspeed[[i]][[j]]^2))))-length(Sspeed[[i]][[j]])*log(norm.const)
        }
      }
    }
  }
  return(ll)
}
neg.logL<-function(par,degree,Tspeed,Sspeed){-logL(par,degree,Tspeed,Sspeed)}

#note: logL for intercept only model is zero 
Fit.lin<-optim(par=c(-6,.3),fn=neg.logL,degree=1,Tspeed=Tspeed,Sspeed=Sspeed) #to run ML, make sure to use -logL
library('numDeriv')
Hess=hessian(func=logL,x=Fit.lin$par,degree=1,Sspeed=Sspeed,Tspeed=Tspeed)
VC=solve(-Hess)
se.lin=sqrt(VC[2,2])
Fit.quad<-optim(par=c(-6,.1,0),fn=neg.logL,degree=2,Tspeed=Tspeed,Sspeed=Sspeed) #to run ML, make sure to use -logL

#MCMC strike rate estimation
old.logL=logL(par=c(MCMC[1,1],MCMC[2,1]),degree=1,Tspeed=Tspeed,Sspeed=Sspeed)
Accept=rep(0,2)
for(iiter in 2:niter){
  if(iiter%%100==1)cat(paste("Iteration ",iiter,"of",niter,'\n'))
  MCMC[,iiter]=MCMC[,iiter-1]  
  ##update alpha0
  alpha0.prop=MCMC[1,iiter-1]+runif(1,-sd.1,sd.1)
  new.logL=logL(par=c(alpha0.prop,MCMC[2,iiter]),degree=1,Tspeed=Tspeed,Sspeed=Sspeed)
  if(runif(1)<exp(new.logL+dnorm(alpha0.prop,0,100,log=TRUE)-old.logL-dnorm(MCMC[1,iiter],0,100,log=TRUE))){
    old.logL=new.logL
    MCMC[1,iiter]=alpha0.prop
    Accept[1]=Accept[1]+1
  }  
  #update alpha1
  alpha1.prop=MCMC[2,iiter-1]+runif(1,-sd.2,sd.2)
  new.logL=logL(par=c(MCMC[1,iiter],alpha1.prop),degree=1,Tspeed=Tspeed,Sspeed=Sspeed)
  if(runif(1)<exp(new.logL+dnorm(alpha1.prop,0,100,log=TRUE)-old.logL-dnorm(MCMC[2,iiter],0,100,log=TRUE))){
    old.logL=new.logL
    MCMC[2,iiter]=alpha1.prop
    Accept[2]=Accept[2]+1
  }  
}
MCMC=MCMC[,(burnin+1):niter]  

tiff('ConnMS13-00004_fig4.tif',height=8,width=8,units='in',compression="lzw",res=250)
par(las=1)
hist(MCMC[2,],xlim=c(0,4),freq=FALSE,xlab=expression(hat(alpha)[1]),ylab="Probability density",cex.axis=1.4,cex.lab=1.5,main=NULL)
dev.off()

###  Generate posterior risk predictions by year and treatment
n.iter=10000
Risk=array(0,dim=c(2,4,n.iter))  #control/treatment, year, iterations

#realized risk
Ref=Transits[which(Transits[,"sma_cycle"]==1 & Transits[,"sma_status"]=="Inactive"),]
Z=sample(Transits[,"transit_length"],19345,replace=TRUE)
Speed.ref=sample(Ref[,"transit_avg_sog"],19345,replace=TRUE)
Speed.ref[Speed.ref>22.5]=22.5
for(itype in 1:2){ #control/treatment
  for(iyear in 1:4){
    if(!((itype==1 & iyear==1)|(itype==1 & iyear==4))){  #don't calculate risk reduction for control period in first or fourth year 
      if(itype==1)status="Inactive"
      if(itype==2)status="Active"        
      Cur.dat=Transits[which(Transits[,"sma_cycle"]==iyear & Transits[,"sma_status"]==status),]
      #sample values of beta
      Cur.beta=Beta[,sample(c(1:n.iter),n.iter,replace=TRUE)]
      Cur.speed=sample(Cur.dat[,"transit_avg_sog"],19345,replace=TRUE)
      Cur.speed[Cur.speed>22.5]=22.5
      Delta.cur=Z/Cur.speed
      Delta.ref=Z/Speed.ref
      Cur.alpha=sample(MCMC[2,],n.iter,replace=TRUE)
      for(iiter in 1:n.iter){
        alpha=Cur.alpha[iiter]
        M.cur=pnorm(Cur.beta[1,iiter]+Cur.beta[2,iiter]*Cur.speed)
        M.ref=pnorm(Cur.beta[1,iiter]+Cur.beta[2,iiter]*Speed.ref)          
        Risk[itype,iyear,iiter]=sum(Delta.cur*M.cur*exp(-20+alpha*Cur.speed))/sum(Delta.ref*M.ref*exp(-20+alpha*Speed.ref))
      }
    }
  }
}

#try to track down how the risk ratio changes as a function of alpha
Cur.alpha=c(1:40)/10
Res=Cur.alpha
Res2=Res
for(i in 1:40){
  alpha=Cur.alpha[i]
  Res[i]=sum(Delta.cur*M.cur*exp(-20+alpha*Cur.speed))/sum(Delta.ref*M.ref*exp(-20+alpha*Speed.ref))
  Res[i]=sum(exp(alpha*Cur.speed))
  Res2[i]=sum(exp(alpha*Speed.ref))
}

#Pooled analysis: realized risk
Ref=Transits[which(Transits[,"sma_status"]=="Inactive"),]
Treat=Transits[which(Transits[,"sma_status"]=="Active"),]
Z=sample(Transits[,"transit_length"],19345,replace=TRUE)
Speed.ref=sample(Ref[,"transit_avg_sog"],19345,replace=TRUE)
Speed.ref[Speed.ref>22.5]=22.5
Cur.speed=sample(Treat[,"transit_avg_sog"],19345,replace=TRUE)
Cur.speed[Cur.speed>22.5]=22.5
Cur.beta=Beta[,sample(c(1:n.iter),n.iter,replace=TRUE)]
Delta.cur=Z/Cur.speed
Delta.ref=Z/Speed.ref
Risk.pooled=rep(0,n.iter)
Cur.alpha=sample(MCMC[2,],n.iter,replace=TRUE)
for(iiter in 1:n.iter){
    alpha=Cur.alpha[iiter]
    M.cur=pnorm(Cur.beta[1,iiter]+Cur.beta[2,iiter]*Cur.speed)
    M.ref=pnorm(Cur.beta[1,iiter]+Cur.beta[2,iiter]*Speed.ref)          
    Risk.pooled[iiter]=sum(Delta.cur*M.cur*exp(-20+alpha*Cur.speed))/sum(Delta.ref*M.ref*exp(-20+alpha*Speed.ref))
}
jpeg('risk_pooled.jpg')
par(las=1)
hist(Risk.pooled,xlim=c(0,0.6),freq=FALSE,xlab='Pooled risk ratio',cex.axis=1.3,cex.lab=1.3,main=NULL)
dev.off()


  
#plot results
require(ggplot2)
#change Risk array into data frame for ggplot
Risk.df=data.frame(matrix(0,n.iter*6,3)) #6 treatment/year combos
colnames(Risk.df)=c("Regulation","Year","value")
cur.iter=1
itype=1
for(iyear in 2:3){
  Risk.df[cur.iter:(cur.iter+n.iter-1),"Regulation"]="N"
  Risk.df[cur.iter:(cur.iter+n.iter-1),"Year"]=2008+iyear
  Risk.df[cur.iter:(cur.iter+n.iter-1),"value"]=Risk[itype,iyear,]
  cur.iter=cur.iter+n.iter
}
itype=2
for(iyear in 1:4){
  Risk.df[cur.iter:(cur.iter+n.iter-1),"Regulation"]="Y"
  Risk.df[cur.iter:(cur.iter+n.iter-1),"Year"]=2008+iyear
  Risk.df[cur.iter:(cur.iter+n.iter-1),"value"]=Risk[itype,iyear,]
  cur.iter=cur.iter+n.iter   
}

Risk.df[,"Regulation"]=as.factor(Risk.df[,"Regulation"])
Risk.df[,"Year"]=as.factor(Risk.df[,"Year"])
cbbPalette <- c("#000000", "#E69F00", "#56B4E9", "#009E73", "#F0E442", "#0072B2", "#D55E00", "#CC79A7")

#plot1=ggplot(Risk.df.treat) + geom_line(aes(x = value,colour = Sensitivity),stat='density',size=1) + labs(x = 'Mortality risk ratio',y="Probability density") + facet_wrap(~Year+Regulation,scales="free_y")
tiff('ConnMS13-00005_fig4.tif',height=8,width=8,units='in',compression="lzw",res=250)
par(las=1)
plot1=ggplot(Risk.df) + geom_line(aes(x = value,colour=Year),stat='density',size=1) + labs(x = 'Mortality risk ratio',y="Probability density") + facet_wrap(~Regulation,scales="free")+scale_colour_manual(values=cbbPalette)
plot1+opts(axis.text.x=theme_text(size=15),axis.text.y=theme_text(size=15),strip.text.x=theme_text(size=16),legend.text=theme_text(size=16),axis.title.x=theme_text(size=18),axis.title.y=theme_text(size=18,angle=90),legend.title=theme_text(size=16))
dev.off()

  
  
